#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_cellular_uplinks_bands_masks_update
short_description: Resource module for devices _cellular _uplinks _bands _masks _update
description:
  - Manage operation create of the resource devices _cellular _uplinks _bands _masks
    _update.
  - Update the cellular band masks for a device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  masked:
    description: Required parameter for the band identifiers to mask for the given
      SIM slot and signal type. For LTE use bands identifiers like '30', for 5G use
      band identifiers like 'n30', or use 'all' to mask all bands for that signal
      type. Maximum 256 bands.
    elements: str
    type: list
  serial:
    description: Serial path parameter.
    type: str
  slot:
    description: Required parameter for the SIM slot to update the cellular band mask
      for.
    type: str
  type:
    description: Required parameter for the signal type to update the cellular band
      mask for.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices createDeviceCellularUplinksBandsMasksUpdate
    description: Complete reference of the createDeviceCellularUplinksBandsMasksUpdate
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-cellular-uplinks-bands-masks-update
notes:
  - SDK Method used are
    devices.Devices.create_device_cellular_uplinks_bands_masks_update,
  - Paths used are
    post /devices/{serial}/cellular/uplinks/bands/masks/update,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_cellular_uplinks_bands_masks_update:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    masked:
      - '2'
      - '12'
      - '30'
    serial: string
    slot: sim1
    type: LTE
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "bySlot": [
        {
          "slot": "string",
          "bySignalType": [
            {
              "type": "string",
              "masked": [
                "string"
              ],
              "enabled": [
                "string"
              ],
              "supported": [
                "string"
              ]
            }
          ]
        }
      ]
    }
"""



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c) 2018, Simon Weald <ansible@simonweald.com>
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import urllib.error as urllib_error
from http import HTTPStatus
from urllib.parse import urlencode

from ansible.module_utils.basic import json
from ansible.module_utils.urls import open_url


class Response:
    """
    Create a response object to mimic that of requests.
    """

    def __init__(self):
        self.content = None
        self.status_code = None
        self.stderr = None

    def json(self):
        return json.loads(self.content)


def memset_api_call(api_key, api_method, payload=None):
    """
    Generic function which returns results back to calling function.

    Requires an API key and an API method to assemble the API URL.
    Returns response text to be analysed.
    """
    # instantiate a response object
    response = Response()

    # if we've already started preloading the payload then copy it
    # and use that, otherwise we need to isntantiate it.
    if payload is None:
        payload = dict()
    else:
        payload = payload.copy()

    # set some sane defaults
    has_failed = False
    msg = None

    data = urlencode(payload)
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    api_uri_base = "https://api.memset.com/v1/json/"
    api_uri = f"{api_uri_base}{api_method}/"

    try:
        resp = open_url(api_uri, data=data, headers=headers, method="POST", force_basic_auth=True, url_username=api_key)
        response.content = resp.read().decode("utf-8")
        response.status_code = resp.getcode()
    except urllib_error.HTTPError as e:
        try:
            errorcode = e.code
        except AttributeError:
            errorcode = None

        has_failed = True
        response.content = e.read().decode("utf8")
        response.status_code = errorcode

        if response.status_code is not None:
            msg = f"Memset API returned a {response.status_code} response ({response.json()['error_type']}, {response.json()['error']})."
        else:
            msg = f"Memset API returned an error ({response.json()['error_type']}, {response.json()['error']})."
    except urllib_error.URLError as e:
        has_failed = True
        msg = f"An URLError occurred ({type(e)})."
        response.stderr = f"{e}"

    if msg is None:
        msg = response.json()

    return has_failed, msg, response


def check_zone_domain(data, domain):
    """
    Returns true if domain already exists, and false if not.
    """
    exists = False

    if data.status_code in (HTTPStatus.CREATED, HTTPStatus.OK):
        for zone_domain in data.json():
            if zone_domain["domain"] == domain:
                exists = True

    return exists


def check_zone(data, name):
    """
    Returns true if zone already exists, and false if not.
    """
    counter = 0
    exists = False

    if data.status_code in (HTTPStatus.CREATED, HTTPStatus.OK):
        for zone in data.json():
            if zone["nickname"] == name:
                counter += 1
        if counter == 1:
            exists = True

    return exists, counter


def get_zone_id(zone_name, current_zones):
    """
    Returns the zone's id if it exists and is unique
    """
    zone_exists = False
    zone_id, msg = None, None
    zone_list = []

    for zone in current_zones:
        if zone["nickname"] == zone_name:
            zone_list.append(zone["id"])

    counter = len(zone_list)

    if counter == 0:
        msg = "No matching zone found"
    elif counter == 1:
        zone_id = zone_list[0]
        zone_exists = True
    elif counter > 1:
        zone_id = None
        msg = "Zone ID could not be returned as duplicate zone names were detected"

    return zone_exists, msg, counter, zone_id



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_show_servers_and_processes
short_description: Shows the status of all processes in the current machine (Multi-Domain Server and all Domain Management / Log Servers). <br>This command is
                   available only on Multi-Domain Server.
description:
  - Shows the status of all processes in the current machine (Multi-Domain Server and all Domain Management / Log Servers). <br>This command is available
    only on Multi-Domain Server.
  - All operations are performed over Web Services API.
  - Available from R81.10 JHF management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options: {}
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: show-servers-and-processes
  cp_mgmt_show_servers_and_processes:
"""

RETURN = """
cp_mgmt_show_servers_and_processes:
  description: The checkpoint show-servers-and-processes output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict()
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "show-servers-and-processes"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard files documentation fragment
    DOCUMENTATION = r"""
attributes:
  idempotent:
    support: full
options: {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_routing_table_lookups_info
short_description: Information module for devices _live _tools _routing _table _lookups
description:
  - Get devices _live _tools _routing _table _lookups by id. - > Return a routing
    table live tool lookup job for a device. The response contains the details of
    the job, including the filters used for the routing table lookup request and the
    results of the request if the job has completed. Only Cisco Secure Routers are
    supported.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
      - Serial path parameter.
    type: str
  id:
    description:
      - Id path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices getDeviceLiveToolsRoutingTableLookup
    description: Complete reference of the getDeviceLiveToolsRoutingTableLookup API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-live-tools-routing-table-lookup
notes:
  - SDK Method used are
    devices.Devices.get_device_live_tools_routing_table_lookup,
  - Paths used are
    get /devices/{serial}/liveTools/routingTable/lookups/{id},
"""

EXAMPLES = r"""
- name: Get devices _live _tools _routing _table _lookups by id
  cisco.meraki.devices_live_tools_routing_table_lookups_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serial: string
    id: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "lookupId": "string",
      "url": "string",
      "request": {
        "serial": "string",
        "type": "string",
        "destination": {
          "address": "string",
          "subnet": "string"
        },
        "nextHop": {
          "address": "string"
        },
        "vpn": {
          "peer": {
            "id": "string"
          }
        }
      },
      "status": "string",
      "entries": [
        {
          "type": "string",
          "subnet": "string",
          "nextHops": [
            {
              "number": 0,
              "address": "string",
              "vpn": {
                "peer": {
                  "id": "string",
                  "name": "string"
                }
              }
            }
          ],
          "ipVersion": "string",
          "vrf": {
            "name": "string"
          }
        }
      ],
      "errors": [
        "string"
      ]
    }
"""



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Simplified BSD License (see simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.module_utils.common.text.converters import to_native
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    get_server_implementation,
    get_server_version,
)


MYSQL_TABLESPACES_MIN_VERSION = (5, 7, 6)
MYSQL_TABLESPACE_ENCRYPTION_MIN_VERSION = (8, 0, 13)
MYSQL_TABLESPACE_STATE_MIN_VERSION = (8, 0, 14)
MYSQL_80_METADATA_FAMILY_MIN_VERSION = (8, 0, 0)


def get_server_version_tuple(cursor):
    version = get_server_version(cursor).split('-', 1)[0]
    version_tuple = []

    for part in version.split('.'):
        digits = ''.join(char for char in part if char.isdigit())
        if not digits:
            break
        version_tuple.append(int(digits))

    while len(version_tuple) < 3:
        version_tuple.append(0)

    return tuple(version_tuple[:3])


def ensure_tablespaces_supported(module, cursor):
    if get_server_implementation(module, cursor) != 'mysql':
        module.fail_json(msg='Tablespace operations are supported only by MySQL.')

    server_version = get_server_version_tuple(cursor)
    if server_version < MYSQL_TABLESPACES_MIN_VERSION:
        module.fail_json(
            msg='Tablespace operations require MySQL %s or later.'
            % _format_version(MYSQL_TABLESPACES_MIN_VERSION)
        )

    return server_version


def get_mysql_tablespaces(cursor, server_version, name=None, module=None):
    if server_version < MYSQL_80_METADATA_FAMILY_MIN_VERSION:
        query = _get_mysql_57_tablespaces_query()
    else:
        query = _get_mysql_80_tablespaces_query(server_version)

    params = None
    if name is not None:
        query += ' AND f.TABLESPACE_NAME = %s'
        params = (name,)

    query += _get_mysql_tablespaces_group_by(server_version)
    return _fetch_normalized_rows(cursor, query, params, module=module)


def get_mysql_tablespace(cursor, server_version, name, module=None):
    tablespaces = get_mysql_tablespaces(cursor, server_version, name=name, module=module)
    if not tablespaces:
        return None
    return tablespaces[0]


def _normalize_mysql_tablespace_row(row):
    return {
        'server_implementation': 'mysql',
        'name': row['TABLESPACE_NAME'],
        'space_id': _to_int_or_none(row.get('FILE_ID')),
        'engine': row['ENGINE'],
        'file_type': row.get('FILE_TYPE'),
        'extent_size': _to_int_or_none(row.get('EXTENT_SIZE')),
        'autoextend_size': _to_int_or_none(row.get('AUTOEXTEND_SIZE')),
        'maximum_size': _to_int_or_none(row.get('MAXIMUM_SIZE')),
        'datafile': row.get('FILE_NAME'),
        'filesystem_block_size': _to_int_or_none(row.get('FS_BLOCK_SIZE')),
        'status': row.get('STATUS'),
        'comment': row.get('EXTRA'),
        'page_size': _to_int_or_none(row.get('PAGE_SIZE')),
        'file_size': _to_int_or_none(row.get('FILE_SIZE')),
        'allocated_size': _to_int_or_none(row.get('ALLOCATED_SIZE')),
        'zip_page_size': _to_int_or_none(row.get('ZIP_PAGE_SIZE')),
        'space_type': row.get('SPACE_TYPE'),
        'state': row.get('STATE'),
        'encryption': row.get('ENCRYPTION'),
        'attached_tables': _to_list_or_empty(row.get('ATTACHED_TABLES')),
    }


def _fetch_normalized_rows(cursor, query, params, module=None):
    try:
        if params:
            cursor.execute(query, params)
        else:
            cursor.execute(query)

        rows = cursor.fetchall()
    except Exception as e:
        if module is not None:
            module.fail_json(msg="Cannot execute SQL '%s': %s" % (query, to_native(e)))
        raise

    return [
        _normalize_mysql_tablespace_row(row)
        for row in rows
    ]


def _get_mysql_57_tablespaces_query():
    return (
        'SELECT f.FILE_ID, f.TABLESPACE_NAME, COALESCE(df.PATH, f.FILE_NAME) AS FILE_NAME, '
        'f.FILE_TYPE, f.ENGINE, f.EXTENT_SIZE, f.AUTOEXTEND_SIZE, f.MAXIMUM_SIZE, '
        'f.STATUS, f.EXTRA, ts.FS_BLOCK_SIZE, ts.FILE_SIZE, ts.ALLOCATED_SIZE, '
        'ts.PAGE_SIZE, ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE, '
        "NULL AS ENCRYPTION, NULL AS STATE, GROUP_CONCAT(t.NAME ORDER BY t.NAME SEPARATOR ',') "
        'AS ATTACHED_TABLES '
        'FROM INFORMATION_SCHEMA.FILES AS f '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_SYS_TABLESPACES AS ts ON ts.SPACE = f.FILE_ID '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_SYS_DATAFILES AS df ON df.SPACE = ts.SPACE '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_SYS_TABLES AS t ON t.SPACE = ts.SPACE '
        "WHERE f.ENGINE = 'InnoDB' AND f.FILE_TYPE = 'TABLESPACE' AND ts.SPACE_TYPE = 'General'"
    )


def _get_mysql_80_tablespaces_query(server_version):
    return (
        'SELECT f.FILE_ID, f.TABLESPACE_NAME, COALESCE(df.PATH, f.FILE_NAME) AS FILE_NAME, '
        'f.FILE_TYPE, f.ENGINE, f.EXTENT_SIZE, f.AUTOEXTEND_SIZE, f.MAXIMUM_SIZE, '
        'f.STATUS, f.EXTRA, ts.FS_BLOCK_SIZE, ts.FILE_SIZE, '
        'ts.ALLOCATED_SIZE, ts.PAGE_SIZE, '
        '%s'
        "GROUP_CONCAT(t.NAME ORDER BY t.NAME SEPARATOR ',') AS ATTACHED_TABLES "
        'FROM INFORMATION_SCHEMA.FILES AS f '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_TABLESPACES AS ts ON ts.SPACE = f.FILE_ID '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_DATAFILES AS df ON df.SPACE = ts.SPACE '
        'LEFT JOIN INFORMATION_SCHEMA.INNODB_TABLES AS t ON t.SPACE = ts.SPACE '
        "WHERE f.ENGINE = 'InnoDB' AND f.FILE_TYPE = 'TABLESPACE' AND ts.SPACE_TYPE = 'General'"
    ) % _get_mysql_80_tablespaces_metadata_columns(server_version)


def _get_mysql_tablespaces_group_by(server_version):
    if server_version < MYSQL_80_METADATA_FAMILY_MIN_VERSION:
        return (
            ' GROUP BY f.FILE_ID, f.TABLESPACE_NAME, df.PATH, f.FILE_NAME, f.FILE_TYPE, '
            'f.ENGINE, f.EXTENT_SIZE, f.AUTOEXTEND_SIZE, f.MAXIMUM_SIZE, '
            'f.STATUS, f.EXTRA, ts.FS_BLOCK_SIZE, ts.FILE_SIZE, ts.ALLOCATED_SIZE, '
            'ts.PAGE_SIZE, ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE ORDER BY f.TABLESPACE_NAME'
        )

    query = (
        ' GROUP BY f.FILE_ID, f.TABLESPACE_NAME, df.PATH, f.FILE_NAME, f.FILE_TYPE, '
        'f.ENGINE, f.EXTENT_SIZE, f.AUTOEXTEND_SIZE, f.MAXIMUM_SIZE, '
        'f.STATUS, f.EXTRA, ts.FS_BLOCK_SIZE, ts.FILE_SIZE, '
        'ts.ALLOCATED_SIZE, ts.PAGE_SIZE, ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE'
    )

    if server_version >= MYSQL_TABLESPACE_ENCRYPTION_MIN_VERSION:
        query += ', ts.ENCRYPTION'

    if server_version >= MYSQL_TABLESPACE_STATE_MIN_VERSION:
        query += ', ts.STATE'

    return query + ' ORDER BY f.TABLESPACE_NAME'


def _get_mysql_80_tablespaces_metadata_columns(server_version):
    if server_version < MYSQL_TABLESPACE_ENCRYPTION_MIN_VERSION:
        return 'ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE, NULL AS ENCRYPTION, NULL AS STATE, '

    if server_version < MYSQL_TABLESPACE_STATE_MIN_VERSION:
        return 'ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE, ts.ENCRYPTION, NULL AS STATE, '

    return 'ts.ZIP_PAGE_SIZE, ts.SPACE_TYPE, ts.ENCRYPTION, ts.STATE, '


def _first_defined(row, *keys):
    for key in keys:
        if key in row and row[key] is not None:
            return row[key]
    return None


def _to_int_or_none(value):
    if value is None:
        return None
    return int(value)


def _to_list_or_empty(value):
    if value is None:
        return []
    if isinstance(value, (list, tuple)):
        return list(value)
    return [item.strip() for item in value.split(',') if item.strip()]


def _format_version(version):
    return '.'.join(str(part) for part in version)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_show_gateways_and_servers
short_description: Shows list of Gateways & Servers sorted by name.
description:
  - Shows list of Gateways & Servers sorted by name.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  limit:
    description:
      - The maximal number of returned results.
        This parameter is relevant only for getting few objects.
    type: int
  offset:
    description:
      - Number of the results to initially skip.
        This parameter is relevant only for getting few objects.
    type: int
  order:
    description:
      - Sorts the results by search criteria. Automatically sorts the results by Name, in the ascending order.
        This parameter is relevant only for getting few objects.
    type: list
    elements: dict
    suboptions:
      ASC:
        description:
          - Sorts results by the given field in ascending order.
        type: str
        choices: ['name']
      DESC:
        description:
          - Sorts results by the given field in descending order.
        type: str
        choices: ['name']
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
      - Available from R81 management version.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: show-gateways-and-servers
  cp_mgmt_show_gateways_and_servers:
    details_level: full
"""

RETURN = """
cp_mgmt_show_gateways_and_servers:
  description: The checkpoint show-gateways-and-servers output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        limit=dict(type='int'),
        offset=dict(type='int'),
        order=dict(type='list', elements='dict', options=dict(
            ASC=dict(type='str', choices=['name']),
            DESC=dict(type='str', choices=['name'])
        )),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "show-gateways-and-servers"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ArgumentSpec,
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.base import (
    DNSConversionError,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.converter import (
    RecordConverter,
)
from ansible_collections.community.dns.plugins.module_utils._options import (
    create_record_transformation_argspec,
)
from ansible_collections.community.dns.plugins.module_utils._record import (
    format_records_for_output,
)
from ansible_collections.community.dns.plugins.module_utils._record_set import (
    format_record_set_for_output,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    NOT_PROVIDED,
    DNSAPIAuthenticationError,
    DNSAPIError,
    ZoneRecordAPI,
)

from ._utils import (
    create_zone_name_id_argspec,
    get_prefix,
    get_zone_id_or_name_with_prefix_filter,
    normalize_dns_name,
)

if t.TYPE_CHECKING:  # pragma: no cover
    from ansible.module_utils.basic import AnsibleModule

    from .._provider import ProviderInformation
    from .._record import (
        DNSRecord,
        RecordIDT,
    )
    from .._record_set import RecordSetIDT
    from .._zone import ZoneIDT
    from .._zone_record_api import NotProvidedType
    from .._zone_record_set_api import ZoneRecordSetAPI


def create_module_argument_spec(
    provider_information: ProviderInformation,
) -> ArgumentSpec:
    return (
        ArgumentSpec(
            argument_spec={
                "what": {
                    "type": "str",
                    "choices": ["single_record", "all_types_for_record", "all_records"],
                    "default": "single_record",
                },
                "record": {"type": "str"},
                "prefix": {"type": "str"},
                "type": {
                    "type": "str",
                    "choices": provider_information.get_supported_record_types(),
                    "default": None,
                },
            },
            required_if=[
                ("what", "single_record", ["type"]),
                ("what", "single_record", ["record", "prefix"], True),
                ("what", "all_types_for_record", ["record", "prefix"], True),
            ],
            mutually_exclusive=[
                ("record", "prefix"),
            ],
        )
        .merge(create_record_transformation_argspec())
        .merge(create_zone_name_id_argspec(provider_information))
    )


def _run_module_record_api(
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    filter_record_type: str | NotProvidedType,
    api: ZoneRecordAPI[ZoneIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    zone_name_in, zone_id_in, filter_prefix = get_zone_id_or_name_with_prefix_filter(
        module.params,
        provider_information,
        use_prefix_or_record=module.params["what"] != "all_records",
    )
    if provider_information.is_zone_id_equal_to_zone_name():
        zone_records = api.get_zone_records(
            zone_id_in,  # type: ignore
            prefix=filter_prefix,
            record_type=filter_record_type,
        )
        if zone_records is None:
            module.fail_json(msg="Zone not found")
        assert zone_name_in is not None
        zone_in = zone_name_in
        zone_id: ZoneIDT = zone_name_in  # type: ignore
    elif zone_name_in is not None:
        zone_in = zone_name_in
        zone = api.get_zone_with_records_by_name(
            zone_in, prefix=filter_prefix, record_type=filter_record_type
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_records = zone.records
        zone_id = zone.zone.id
    else:
        zone = api.get_zone_with_records_by_id(
            zone_id_in,  # type: ignore
            prefix=filter_prefix,
            record_type=filter_record_type,
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        zone_records = zone.records
        zone_id = zone.zone.id

    # Retrieve requested information
    if module.params["what"] == "single_record":
        # Extract prefix
        record_in = normalize_dns_name(module.params.get("record"))
        prefix_in = module.params.get("prefix")
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )

        # Find matching records
        records_list = []
        for record in zone_records:
            if record.prefix == prefix:
                records_list.append(record)

        # Convert records
        record_converter.process_multiple_from_api(records_list)
        record_converter.process_multiple_to_user(records_list)

        # Format output
        data = (
            format_records_for_output(records_list, record_in, prefix)
            if records_list
            else {}
        )
        module.exit_json(
            changed=False,
            set=data,
            zone_id=zone_id,
        )
    else:
        # Extract prefix if necessary
        if module.params["what"] == "all_types_for_record":
            check_prefix = True
            record_in = normalize_dns_name(module.params.get("record"))
            prefix_in = module.params.get("prefix")
            record_in, prefix = get_prefix(
                normalized_zone=zone_in,
                normalized_record=record_in,
                prefix=prefix_in,
                provider_information=provider_information,
            )
        else:
            check_prefix = False
            prefix = None

        # Find matching records
        records: dict[tuple[str, str], list[DNSRecord]] = {}
        for record in zone_records:
            if check_prefix and record.prefix != prefix:
                continue
            key = (
                (record.prefix + "." + zone_in) if record.prefix else zone_in,
                record.type,
            )
            record_list = records.get(key)
            if record_list is None:
                record_list = records[key] = []
            record_list.append(record)

        # Convert records
        for record_list in records.values():
            record_converter.process_multiple_from_api(record_list)
            record_converter.process_multiple_to_user(record_list)

        # Format output
        data2 = [
            format_records_for_output(record_list, record_name, record_list[0].prefix)
            for (record_name, dummy), record_list in sorted(records.items())
        ]
        module.exit_json(
            changed=False,
            sets=data2,
            zone_id=zone_id,
        )


def _run_module_record_set_api(
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    filter_record_type: str | NotProvidedType,
    api: ZoneRecordSetAPI[ZoneIDT, RecordSetIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    zone_name_in, zone_id_in, filter_prefix = get_zone_id_or_name_with_prefix_filter(
        module.params,
        provider_information,
        use_prefix_or_record=module.params["what"] != "all_records",
    )
    if provider_information.is_zone_id_equal_to_zone_name():
        zone_record_sets = api.get_zone_record_sets(
            zone_id_in,  # type: ignore
            prefix=filter_prefix,
            record_type=filter_record_type,
        )
        if zone_record_sets is None:
            module.fail_json(msg="Zone not found")
        assert zone_name_in is not None
        zone_in = zone_name_in
        zone_id: ZoneIDT = zone_name_in  # type: ignore
    elif zone_name_in is not None:
        zone_in = zone_name_in
        zone = api.get_zone_with_record_sets_by_name(
            zone_in, prefix=filter_prefix, record_type=filter_record_type
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_record_sets = zone.record_sets
        zone_id = zone.zone.id
    else:
        zone = api.get_zone_with_record_sets_by_id(
            zone_id_in,  # type: ignore
            prefix=filter_prefix,
            record_type=filter_record_type,
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        zone_record_sets = zone.record_sets
        zone_id = zone.zone.id

    # Retrieve requested information
    if module.params["what"] == "single_record":
        # Extract prefix
        record_in = normalize_dns_name(module.params.get("record"))
        prefix_in = module.params.get("prefix")
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )

        # Find matching record set
        record_set = next(
            (
                candidate_record_set
                for candidate_record_set in zone_record_sets
                if candidate_record_set.prefix == prefix
            ),
            None,
        )

        # Convert record set
        if record_set:
            record_converter.process_set_from_api(record_set)
            record_converter.process_set_to_user(record_set)

        # Format output
        data = (
            format_record_set_for_output(record_set, record_in, prefix)
            if record_set
            else {}
        )
        module.exit_json(
            changed=False,
            set=data,
            zone_id=zone_id,
        )
    else:
        # Extract prefix if necessary
        if module.params["what"] == "all_types_for_record":
            check_prefix = True
            record_in = normalize_dns_name(module.params.get("record"))
            prefix_in = module.params.get("prefix")
            record_in, prefix = get_prefix(
                normalized_zone=zone_in,
                normalized_record=record_in,
                prefix=prefix_in,
                provider_information=provider_information,
            )
        else:
            check_prefix = False
            prefix = None

        def get_record_name(record_set):
            return record_set.prefix + "." + zone_in if record_set.prefix else zone_in

        # Find matching records
        record_sets = list(zone_record_sets)
        if check_prefix:
            record_sets = [
                record_set for record_set in record_sets if record_set.prefix == prefix
            ]
        record_sets.sort(
            key=lambda record_set: (get_record_name(record_set), record_set.type)
        )

        # Convert records
        for record_set in record_sets:
            record_converter.process_set_from_api(record_set)
            record_converter.process_set_to_user(record_set)

        # Format output
        data2 = [
            format_record_set_for_output(
                record_set, get_record_name(record_set), record_set.prefix
            )
            for record_set in record_sets
        ]
        module.exit_json(
            changed=False,
            sets=data2,
            zone_id=zone_id,
        )


def run_module(
    module: AnsibleModule,
    create_api: t.Callable[[], ZoneRecordAPI | ZoneRecordSetAPI],
    provider_information: ProviderInformation,
) -> t.NoReturn:
    option_provider = ModuleOptionProvider(module)
    record_converter = RecordConverter(provider_information, option_provider)
    record_converter.emit_deprecations(module.deprecate)

    filter_record_type: str | NotProvidedType = NOT_PROVIDED
    if module.params["what"] == "single_record":
        filter_record_type = module.params.get("type")
        if (
            filter_record_type
            and filter_record_type
            not in provider_information.get_supported_record_types()
        ):
            module.fail_json(msg=f"Invalid record type {filter_record_type}")

    try:
        # Create API
        api = create_api()

        if isinstance(api, ZoneRecordAPI):
            _run_module_record_api(
                module,
                provider_information,
                record_converter,
                filter_record_type,
                api,
            )
        else:
            _run_module_record_set_api(
                module,
                provider_information,
                record_converter,
                filter_record_type,
                api,
            )

    except DNSConversionError as e:
        module.fail_json(
            msg=f"Error while converting DNS values: {e.error_message}",
            error=e.error_message,
            exception=traceback.format_exc(),
        )
    except DNSAPIAuthenticationError as e:
        module.fail_json(
            msg=f"Cannot authenticate: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )
    except DNSAPIError as e:
        module.fail_json(
            msg=f"Error: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )



# Copyright (c) 2026 Felix Fontein
# Copyright (c) 2026 Plexim GmbH
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

# API docs:
# https://developer.infomaniak.com/docs/api/get/2/zones/%7Bzone%7D/records

# **NOTE** that we're using the zone's FQDN as the zone ID, instead of the integer
# value provided in the zone object, since Infomaniak's API does not allow to
# access zone or record data with the integer ID, only with the FQDN.

from __future__ import annotations

import typing as t
from urllib.parse import quote

from ansible_collections.community.dns.plugins.module_utils._argspec import ArgumentSpec
from ansible_collections.community.dns.plugins.module_utils._json_api_helper import (
    JSONAPIHelper,
)
from ansible_collections.community.dns.plugins.module_utils._provider import (
    ProviderInformation,
)
from ansible_collections.community.dns.plugins.module_utils._record import DNSRecord
from ansible_collections.community.dns.plugins.module_utils._zone import (
    DNSZone,
    DNSZoneWithRecords,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    NOT_PROVIDED,
    DNSAPIError,
    ZoneRecordAPI,
    filter_records,
)

if t.TYPE_CHECKING:  # pragma: no cover
    from collections.abc import Sequence

    from .._argspec import OptionProvider
    from .._http import HTTPHelper
    from .._provider import AnsibleType
    from .._record import IDNSRecord
    from .._zone_record_api import NotProvidedType


def _create_record_from_json(source: dict[str, t.Any]) -> DNSRecord[int]:
    record_type = source.get("type")
    if record_type is None:  # pragma: no cover
        raise DNSAPIError("Record from API has no type")
    prefix: str | None = source["source"]
    if prefix in ("", "."):
        prefix = None
    result = DNSRecord(
        record_id=source["id"],
        record_type=record_type,
        target=source["target"],
    )
    result.ttl = source["ttl"]
    result.prefix = prefix
    return result


def _create_zone_from_json(source: dict[str, t.Any]) -> DNSZone[str]:
    # We don't use the "id" field.
    zone = DNSZone(zone_id=source["fqdn"], name=source["fqdn"])
    zone.info = {
        "dnssec": source["dnssec"],  # dict with key "is_enabled"
        "id": source["id"],  # integer
        "nameservers": source["nameservers"],  # list of strings
    }
    return zone


def _create_zone_with_records_from_json(
    source: dict[str, t.Any],
    prefix: str | None | NotProvidedType = NOT_PROVIDED,
    record_type: str | NotProvidedType = NOT_PROVIDED,
) -> DNSZoneWithRecords[str, int]:
    return DNSZoneWithRecords(
        _create_zone_from_json(source),
        filter_records(
            [_create_record_from_json(record) for record in source["records"]],
            prefix=prefix,
            record_type=record_type,
        ),
    )


def _pagination_query(
    query: dict[str, str] | Sequence[tuple[str, str]] | None,
    *,
    per_page: int,
    page: int,
) -> dict[str, str] | Sequence[tuple[str, str]]:
    if isinstance(query, dict):
        query_map = query.copy()
        query_map["per_page"] = str(per_page)
        query_map["page"] = str(page)
        return query_map
    query_list = list(query) if query else []
    query_list.append(("per_page", str(per_page)))
    query_list.append(("page", str(page)))
    return query_list


def q(value: str | int | None) -> str:
    if value is None:
        return ""
    return quote(str(value), safe="", encoding="utf-8", errors=None)


class InfomaniakJSONAPI(ZoneRecordAPI[str, int], JSONAPIHelper):
    def __init__(
        self,
        *,
        http_helper: HTTPHelper,
        token: str,
        api: str = "https://api.infomaniak.com",
        debug: bool = False,
    ) -> None:
        """
        Create a new Infomaniak API instance with given API token.
        """
        JSONAPIHelper.__init__(self, http_helper, token, api=api, debug=debug)

    def _extract_error_message(self, result: t.Any | None) -> str:
        if result is None:
            return ""
        if isinstance(result, dict):
            res = ""
            if "error" in result:
                error = result["error"]
                if (
                    isinstance(error, dict)
                    and isinstance(error.get("code"), str)
                    and isinstance(error.get("description"), str)
                ):
                    res = f"[{error['code']}] {error['description']}"
            if res:
                return res
        return f" with data: {result}"

    def _create_headers(self) -> dict[str, str]:
        return {
            "accept": "application/json",
            "authorization": f"Bearer {self._token}",
        }

    @t.overload
    def _list_pagination(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        per_page: int = 100,
        accept_404: t.Literal[False] = False,
    ) -> list[t.Any]: ...  # pragma: no cover

    @t.overload
    def _list_pagination(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        per_page: int = 100,
        accept_404: t.Literal[True],
    ) -> list[t.Any] | None: ...  # pragma: no cover

    def _list_pagination(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        per_page: int = 100,
        accept_404: bool = False,
    ) -> list[t.Any] | None:
        result = []
        page = 1
        while True:
            query_ = _pagination_query(query, per_page=per_page, page=page)
            res, info = self._get(
                url,
                query=query_,
                must_have_content=True,
                expected=[200, 404] if accept_404 and page == 1 else [200],
                require_json_object=True,
            )
            if info["status"] == 404:
                return None
            result.extend(res["data"])
            if len(res["data"]) < per_page:
                return result
            page += 1

    def get_zone_with_records_by_id(
        self,
        zone_id: str,
        prefix: str | None | NotProvidedType = NOT_PROVIDED,
        record_type: str | NotProvidedType = NOT_PROVIDED,
    ) -> DNSZoneWithRecords[str, int] | None:
        """
        Given a zone ID, return the zone contents with records if found.

        @param zone_id: The zone ID
        @param prefix: The prefix to filter for, if provided. Since None is a valid value,
                       the special constant NOT_PROVIDED indicates that we are not filtering.
        @param record_type: The record type to filter for, if provided
        @return The zone information with records (DNSZoneWithRecords), or None if not found
        """
        return self.get_zone_with_records_by_name(
            zone_id, prefix=prefix, record_type=record_type
        )  # pragma: no cover

    def get_zone_with_records_by_name(
        self,
        name: str,
        prefix: str | None | NotProvidedType = NOT_PROVIDED,
        record_type: str | NotProvidedType = NOT_PROVIDED,
    ) -> DNSZoneWithRecords[str, int] | None:
        """
        Given a zone name, return the zone contents with records if found.

        @param name: The zone name (string)
        @param prefix: The prefix to filter for, if provided. Since None is a valid value,
                       the special constant NOT_PROVIDED indicates that we are not filtering.
        @param record_type: The record type to filter for, if provided
        @return The zone information with records (DNSZoneWithRecords), or None if not found
        """
        result, info = self._get(
            f"/2/zones/{q(name)}?with=records",
            expected=[200, 404],
            must_have_content=[200, 404],
            require_json_object=True,
        )
        if info["status"] == 404 or result is None:
            return None
        return _create_zone_with_records_from_json(
            result["data"], prefix=prefix, record_type=record_type
        )

    def get_zone_records(
        self,
        zone_id: str,
        prefix: str | None | NotProvidedType = NOT_PROVIDED,
        record_type: str | NotProvidedType = NOT_PROVIDED,
    ) -> list[DNSRecord[int]] | None:
        """
        Given a zone ID, return a list of records, optionally filtered by the provided criteria.

        @param zone_id: The zone ID
        @param prefix: The prefix to filter for, if provided. Since None is a valid value,
                       the special constant NOT_PROVIDED indicates that we are not filtering.
        @param record_type: The record type to filter for, if provided
        @return A list of DNSrecord objects, or None if zone was not found
        """
        query: dict[str, str] = {}
        if prefix is not NOT_PROVIDED:
            query["filter[source]"] = prefix or "."  # type: ignore  # Is a string
        if record_type is not NOT_PROVIDED:
            query["filter[types][]"] = record_type.upper()  # type: ignore  # Is a string
        result = self._list_pagination(
            f"/2/zones/{q(zone_id)}/records",
            query=query,
            accept_404=True,
        )
        if result is None:
            return None
        return filter_records(
            [_create_record_from_json(record) for record in result],
            prefix=prefix,
            record_type=record_type,
        )

    def get_zone_by_name(self, name: str) -> DNSZone[str] | None:
        """
        Given a zone name, return the zone contents if found.

        @param name: The zone name (string)
        @return The zone information (DNSZone), or None if not found
        """
        result, info = self._get(
            f"/2/zones/{q(name)}",
            expected=[200, 404],
            must_have_content=[200, 404],
            require_json_object=True,
        )
        if info["status"] == 404 or result is None:
            return None
        return _create_zone_from_json(result["data"])

    def get_zone_by_id(self, zone_id: str) -> DNSZone[str] | None:
        """
        Given a zone ID, return the zone contents if found.

        @param zone_id: The zone ID
        @return The zone information (DNSZone), or None if not found
        """
        return self.get_zone_by_name(zone_id)  # pragma: no cover

    def add_record(
        self, zone_id: str, record: IDNSRecord[int | None]
    ) -> DNSRecord[int]:
        """
        Adds a new record to an existing zone.

        @param zone_id: The zone ID
        @param record: The DNS record (DNSRecord)
        @return The created DNS record (DNSRecord)
        """
        data = {
            "type": record.type,
            "ttl": record.ttl,
            "source": record.prefix or ".",
            "target": record.target,
        }
        result, dummy = self._post(
            f"/2/zones/{q(zone_id)}/records",
            data=data,
            # Documentation says 200, API actually returns 201...
            expected=[200, 201],
            require_json_object=True,
        )
        return _create_record_from_json(result["data"])

    def update_record(self, zone_id: str, record: DNSRecord[int]) -> DNSRecord[int]:
        """
        Update a record.

        @param zone_id: The zone ID
        @param record: The DNS record (DNSRecord)
        @return The DNS record (DNSRecord)
        """
        if record.id is None:
            raise DNSAPIError("Need record ID to update record!")
        data = {
            "ttl": record.ttl,
            "target": record.target,
        }
        result, dummy = self._put(
            f"/2/zones/{q(zone_id)}/records/{q(record.id)}",
            data=data,
            expected=[200],
            require_json_object=True,
        )
        return _create_record_from_json(result["data"])

    def delete_record(self, zone_id: str, record: DNSRecord[int]) -> bool:
        """
        Delete a record.

        @param zone_id: The zone ID
        @param record: The DNS record (DNSRecord)
        @return True in case of success (boolean)
        """
        if record.id is None:
            raise DNSAPIError("Need record ID to delete record!")
        result, info = self._delete(
            f"/2/zones/{q(zone_id)}/records/{q(record.id)}",
            expected=[200, 404],
            require_json_object=True,
        )
        return info["status"] == 200 and result["result"] == "success"


class InfomaniakProviderInformation(ProviderInformation):
    def get_supported_record_types(self) -> list[str]:
        """
        Return a list of supported record types.
        """
        return [
            "A",
            "AAAA",
            "CAA",
            "CNAME",
            "DNAME",
            "DNSKEY",
            "DS",
            "MX",
            "NS",
            "PTR",
            "SMIMEA",
            "SOA",
            "SRV",
            "SSHFP",
            "TLSA",
            "TXT",
        ]

    def get_zone_id_type(self) -> AnsibleType:
        """
        Return the (short) type for zone IDs, like ``'int'`` or ``'str'``.
        """
        return "str"

    def is_zone_id_equal_to_zone_name(self) -> bool:
        """
        Whether the zone ID is equal to the zone's name (FQDN).

        If ``True``, implies that ``get_zone_id_type()`` returns ``str``.
        """
        return True

    def get_record_id_type(self) -> AnsibleType:
        """
        Return the (short) type for record IDs, like ``'int'`` or ``'str'``.
        """
        return "int"  # pragma: no cover

    def get_record_default_ttl(self) -> int | None:
        """
        Return the default TTL for records, like 300, 3600 or None.
        None means that some other TTL (usually from the zone) will be used.
        """
        # There is no default, every record must have a TTL, so we just pick a "random" number.
        return 300

    def normalize_prefix(self, prefix: str | None) -> str | None:
        """
        Given a prefix (string or None), return its normalized form.

        The result should always be None for the trivial prefix, and a non-zero length DNS name
        for a non-trivial prefix.

        If a provider supports other identifiers for the trivial prefix, such as '@', this
        function needs to convert them to None as well.
        """
        return None if prefix in (".", "") else prefix

    def txt_record_handling(
        self,
    ) -> t.Literal["decoded", "encoded", "encoded-no-char-encoding"]:
        """
        Return how the API handles TXT records.

        Returns one of the following strings:
        * 'decoded' - the API works with unencoded values
        * 'encoded' - the API works with encoded values
        * 'encoded-no-char-encoding' - the API works with encoded values, but without character encoding
        """
        return "encoded-no-char-encoding"

    def txt_decode_lenient_from_api(self) -> bool:
        """
        Whether TXT records send from the API should be decoded with lenient=True.
        """
        return True


def create_infomaniak_provider_information(
    option_provider: OptionProvider | None = None,
) -> InfomaniakProviderInformation:
    return InfomaniakProviderInformation()


def create_infomaniak_argument_spec() -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "infomaniak_token": {
                "type": "str",
                "required": True,
                "no_log": True,
                "aliases": ["api_token"],
            },
        },
    )


def create_infomaniak_api(
    option_provider: OptionProvider, http_helper: HTTPHelper
) -> InfomaniakJSONAPI:
    token = option_provider.get_option("infomaniak_token")
    if token is not None:
        return InfomaniakJSONAPI(http_helper=http_helper, token=token)

    raise DNSAPIError("infomaniak_token must be provided!")  # pragma: no cover



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_sm_devices_move
short_description: Resource module for networks _sm _devices _move
description:
  - Manage operation create of the resource networks _sm _devices _move.
  - Move a set of devices to a new network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  ids:
    description: The ids of the devices to be moved.
    elements: str
    type: list
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  newNetwork:
    description: The new network to which the devices will be moved.
    type: str
  scope:
    description: The scope (one of all, none, withAny, withAll, withoutAny, or withoutAll)
      and a set of tags of the devices to be moved.
    elements: str
    type: list
  serials:
    description: The serials of the devices to be moved.
    elements: str
    type: list
  wifiMacs:
    description: The wifiMacs of the devices to be moved.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sm moveNetworkSmDevices
    description: Complete reference of the moveNetworkSmDevices API.
    link: https://developer.cisco.com/meraki/api-v1/#!move-network-sm-devices
notes:
  - SDK Method used are
    sm.Sm.move_network_sm_devices,
  - Paths used are
    post /networks/{networkId}/sm/devices/move,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_sm_devices_move:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    ids:
      - '1284392014819'
      - '2983092129865'
    networkId: string
    newNetwork: '1284392014819'
    scope:
      - withAny
      - tag1
      - tag2
    serials:
      - XY0XX0Y0X0
      - A01B01CD00E
      - X02YZ1ZYZX
    wifiMacs:
      - 00:11:22:33:44:55
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "ids": [
        "string"
      ],
      "newNetwork": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_appliance_interfaces_ports_update
short_description: Resource module for devices _appliance _interfaces _ports _update
description:
  - Manage operation create of the resource devices _appliance _interfaces _ports
    _update.
  - Update configurations for an appliance's specified port.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  downlink:
    description: The port's VLAN settings when in LAN mode.
    suboptions:
      access:
        description: The port's settings when in 'access' mode.
        suboptions:
          policy:
            description: The access policy settings for this port.
            suboptions:
              type:
                description: The access policy that will be enforced by the 'access'
                  VLAN.
                type: str
            type: dict
          vlan:
            description: The VLAN for which this port will accept and pass traffic
              in 'access' mode. All untagged traffic will automatically be treated
              as if it belonged to this VLAN.
            type: str
        type: dict
      mode:
        description: Indicates whether the port is in 'trunk' or 'access' mode.
        type: str
      sgt:
        description: Security Group Tag settings for this port.
        suboptions:
          id:
            description: Adaptive policy group ID that all traffic originating from
              this port is assigned to.
            type: str
        type: dict
      trunk:
        description: The port's settings when in 'trunk' mode.
        suboptions:
          allowedVlans:
            description: The VLANs for which this port will accept and pass traffic
              in 'trunk' mode. This must include the Native VLAN if one is set.
            elements: str
            type: list
          nativeVlan:
            description: The Native VLAN for the port. All untagged traffic that comes
              in on this port will be treated as if it belonged to this VLAN. This
              can also be set to 0 to drop untagged traffic.
            type: str
          sgt:
            description: Security Group Tag settings for this trunk port.
            suboptions:
              enabled:
                description: Indicates whether the trunk port is Peer SGT capable.
                type: bool
            type: dict
        type: dict
    type: dict
  enabled:
    description: Indicates whether the port is enabled.
    type: bool
  interface:
    description: The interface tuple used to identify the port.
    suboptions:
      number:
        description: The leaf port number.
        type: int
      slot:
        description: The slot number for the port.
        type: int
      subslot:
        description: The subslot number for the port.
        type: int
    type: dict
  personality:
    description: Describes the port's configurability.
    suboptions:
      layer:
        description: Describes the port's layer configurability.
        suboptions:
          mode:
            description: The layer at which the port operates.
            type: int
        type: dict
      mode:
        description: The type of interface, 'wan' or 'lan', the port is configured
          as.
        type: str
    type: dict
  serial:
    description: Serial path parameter.
    type: str
  uplink:
    description: The port's settings when in WAN mode.
    suboptions:
      type:
        description: Describes the uplink device.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createDeviceApplianceInterfacesPortsUpdate
    description: Complete reference of the createDeviceApplianceInterfacesPortsUpdate
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-appliance-interfaces-ports-update
notes:
  - SDK Method used are
    appliance.Appliance.create_device_appliance_interfaces_ports_update,
  - Paths used are
    post /devices/{serial}/appliance/interfaces/ports/update,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_appliance_interfaces_ports_update:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    downlink:
      access:
        policy:
          type: 802.1X
        vlan: '1'
      mode: access
      sgt:
        id: '1234'
      trunk:
        allowedVlans:
          - '2'
          - '3'
          - '4'
          - '5'
        nativeVlan: '2'
        sgt:
          enabled: false
    enabled: true
    interface:
      number: 3
      slot: 1
      subslot: 2
    personality:
      layer:
        mode: 3
      mode: wan
    serial: string
    uplink:
      type: ethernet
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "number": "string",
      "interface": {
        "name": "string",
        "slot": {},
        "subslot": {},
        "number": 0
      },
      "enabled": true,
      "name": "string",
      "personality": {
        "mode": "string",
        "isFlexible": true,
        "layer": {
          "mode": 0,
          "isFlexible": true
        }
      },
      "uplink": {
        "type": "string",
        "primary": true
      },
      "downlink": {
        "mode": "string",
        "sgt": {
          "id": {}
        },
        "access": {
          "vlan": "string",
          "policy": {
            "type": {}
          }
        },
        "trunk": {
          "nativeVlan": "string",
          "allowedVlans": [
            "string"
          ],
          "sgt": {
            "enabled": true
          }
        }
      }
    }
"""



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard files documentation fragment

    # NOTE: This document fragment needs to be augmented by ZONE_ID_TYPE in a provider document fragment.
    #       The ZONE_ID_TYPE fragment will provide `choices` for the options.type entry.
    DOCUMENTATION = r"""
attributes:
  idempotent:
    support: full
options:
  what:
    description:
      - Describes whether to fetch a single record and type combination, all types for a record, or all records. By default,
        a single record and type combination is fetched.
      - Note that the return value structure depends on this option.
    choices: ['single_record', 'all_types_for_record', 'all_records']
    default: single_record
    type: str
  record:
    description:
      - The full DNS record to retrieve.
      - If O(what) is V(single_record) or V(all_types_for_record), exactly one of O(record) and O(prefix) is required.
    type: str
  prefix:
    description:
      - The prefix of the DNS record.
      - This is the part of O(record) before O(zone_name). For example, if the record to be modified is C(www.example.com)
        for the zone C(example.com), the prefix is V(www). If the record in this example would be C(example.com), the prefix
        would be V('') (empty string).
      - If O(what) is V(single_record) or V(all_types_for_record), exactly one of O(record) and O(prefix) is required.
    type: str
  type:
    description:
      - The type of DNS record to retrieve.
      - Required if O(what) is V(single_record).
    type: str
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_replication_filter

short_description: Manage MySQL or MariaDB replication filters

description:
  - "Important: MariaDB support will be dropped in collection version 6.0.0. Use the M(ansible.mariadb.mariadb_replication_filter) instead."
  - Manage replica-side replication filters declaratively.
  - MySQL uses C(CHANGE REPLICATION FILTER).
  - MariaDB uses C(SET GLOBAL replicate_*) variables.
  - Changes are runtime only and are not persisted across restart by this module.
  - Persist filter settings separately through server configuration if they must survive a restart.

author:
  - Ron Gershburg (@ronger4)

version_added: '5.2.0'

options:
  replicate_do_db:
    description:
      - Databases that should be replicated.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  replicate_ignore_db:
    description:
      - Databases that should be ignored during replication.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  replicate_do_table:
    description:
      - Fully qualified tables that should be replicated.
      - Values must use C(database.table) syntax.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  replicate_ignore_table:
    description:
      - Fully qualified tables that should be ignored during replication.
      - Values must use C(database.table) syntax.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  replicate_wild_do_table:
    description:
      - Wildcard table patterns that should be replicated.
      - Values must use C(database.table) syntax with C(%) and C(_) wildcards.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  replicate_wild_ignore_table:
    description:
      - Wildcard table patterns that should be ignored during replication.
      - Values must use C(database.table) syntax with C(%) and C(_) wildcards.
      - Provide an empty list to clear the filter.
    type: list
    elements: str
  channel:
    description:
      - MySQL replication channel name.
      - Supported only for MySQL.
    type: str
  connection_name:
    description:
      - MariaDB replication connection name.
      - Supported only for MariaDB multi-source replication.
    type: str

notes:
  - Compatible with MariaDB or MySQL.
  - At least one replication filter option must be provided.
  - Unspecified filters are left unchanged.

attributes:
  check_mode:
    support: full
  idempotent:
    support: full

extends_documentation_fragment:
  - ansible.mysql.mysql
'''

EXAMPLES = r'''
- name: Ignore one database on a MySQL replica
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    replicate_ignore_db:
      - audit

- name: Replicate selected tables on a MySQL replica
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    replicate_do_table:
      - app.orders
      - reporting.summary

- name: Set wildcard filters on a MySQL replica
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    replicate_wild_do_table:
      - app.%
      - reporting.sales_%
    replicate_wild_ignore_table:
      - archive.old_%
      - tmp.%

- name: Set multiple MySQL filters in one task
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    replicate_do_db:
      - app
    replicate_ignore_table:
      - archive.events

- name: Set one channel-specific MySQL filter
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    channel: analytics
    replicate_do_db:
      - reporting

- name: Set one named MariaDB replication filter
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    connection_name: analytics
    replicate_do_db:
      - reporting

- name: Clear one MariaDB wildcard filter
  ansible.mysql.mysql_replication_filter:
    login_unix_socket: /run/mysqld/mysqld.sock
    replicate_wild_ignore_table: []
'''

RETURN = r'''
queries:
  description: Executed SQL statements or predicted SQL statements in check mode.
  returned: always
  type: list
  elements: str
  sample:
    - CHANGE REPLICATION FILTER REPLICATE_DO_DB = (`app`,`reporting`)
filters:
  description: Effective normalized filter state after execution or prediction.
  returned: always
  type: dict
  contains:
    replicate_do_db:
      description: Databases that should be replicated.
      returned: always
      type: list
      elements: str
    replicate_ignore_db:
      description: Databases that should be ignored during replication.
      returned: always
      type: list
      elements: str
    replicate_do_table:
      description: Fully qualified tables that should be replicated.
      returned: always
      type: list
      elements: str
    replicate_ignore_table:
      description: Fully qualified tables that should be ignored during replication.
      returned: always
      type: list
      elements: str
    replicate_wild_do_table:
      description: Wildcard table patterns that should be replicated.
      returned: always
      type: list
      elements: str
    replicate_wild_ignore_table:
      description: Wildcard table patterns that should be ignored during replication.
      returned: always
      type: list
      elements: str
  sample:
    replicate_do_db:
      - app
      - reporting
    replicate_ignore_db: []
    replicate_do_table: []
    replicate_ignore_table: []
    replicate_wild_do_table: []
    replicate_wild_ignore_table: []
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.ansible.mysql.plugins.module_utils.database import check_input, mysql_quote_identifier
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    get_server_implementation,
    get_server_version,
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)
from ansible_collections.ansible.mysql.plugins.module_utils.version import LooseVersion


FILTER_DEFINITIONS = {
    'replicate_do_db': {
        'mysql_name': 'REPLICATE_DO_DB',
        'mariadb_name': 'replicate_do_db',
        'value_kind': 'database',
    },
    'replicate_ignore_db': {
        'mysql_name': 'REPLICATE_IGNORE_DB',
        'mariadb_name': 'replicate_ignore_db',
        'value_kind': 'database',
    },
    'replicate_do_table': {
        'mysql_name': 'REPLICATE_DO_TABLE',
        'mariadb_name': 'replicate_do_table',
        'value_kind': 'table',
    },
    'replicate_ignore_table': {
        'mysql_name': 'REPLICATE_IGNORE_TABLE',
        'mariadb_name': 'replicate_ignore_table',
        'value_kind': 'table',
    },
    'replicate_wild_do_table': {
        'mysql_name': 'REPLICATE_WILD_DO_TABLE',
        'mariadb_name': 'replicate_wild_do_table',
        'value_kind': 'pattern',
    },
    'replicate_wild_ignore_table': {
        'mysql_name': 'REPLICATE_WILD_IGNORE_TABLE',
        'mariadb_name': 'replicate_wild_ignore_table',
        'value_kind': 'pattern',
    },
}

MYSQL_FILTER_NAME_TO_PARAM = dict(
    (definition['mysql_name'], param)
    for param, definition in FILTER_DEFINITIONS.items()
)


def normalize_filter_values(values):
    normalized = []

    for value in values:
        value = value.strip()
        if not value:
            continue
        normalized.append(value)

    return sorted(set(normalized))


def normalize_mysql_filter_rows(rows):
    filters = _empty_filters()

    for row in rows:
        param = MYSQL_FILTER_NAME_TO_PARAM.get(row['FILTER_NAME'])
        if not param:
            continue
        filters[param] = normalize_filter_values(_split_filter_rule(row.get('FILTER_RULE', '')))

    return filters


def normalize_mariadb_filter_values(variables):
    filters = _empty_filters()

    for param, definition in FILTER_DEFINITIONS.items():
        raw_value = variables.get(definition['mariadb_name'], '')
        filters[param] = normalize_filter_values(_split_filter_rule(raw_value))

    return filters


def plan_filter_changes(desired_filters, current_filters, query_builder):
    planned_filters = dict((name, list(values)) for name, values in current_filters.items())
    queries = []

    for param, desired_values in desired_filters.items():
        if desired_values is None:
            continue

        normalized_values = normalize_filter_values(desired_values)
        planned_filters[param] = normalized_values

        if current_filters.get(param, []) != normalized_values:
            queries.append(query_builder(param, normalized_values))

    return {
        'changed': bool(queries),
        'queries': queries,
        'filters': planned_filters,
    }


def build_mysql_filter_query(filter_name, value_kind, values, channel=''):
    values_sql = _build_mysql_filter_values(value_kind, values)
    query = 'CHANGE REPLICATION FILTER %s = (%s)' % (filter_name, values_sql)

    if channel:
        query += ' FOR CHANNEL %s' % _quote_sql_string(channel)

    return {
        'sql': query,
        'params': (),
        'display': query,
    }


def build_mariadb_set_query(variable_name, values):
    value = ','.join(normalize_filter_values(values))
    identifier = mysql_quote_identifier(variable_name, 'vars')
    query = 'SET GLOBAL %s = %%s' % identifier

    return {
        'sql': query,
        'params': (value,),
        'display': 'SET GLOBAL %s = %s' % (identifier, _quote_sql_string(value)),
    }


def build_mysql_query_from_param(param, values, channel=''):
    definition = FILTER_DEFINITIONS[param]
    return build_mysql_filter_query(
        definition['mysql_name'],
        definition['value_kind'],
        values,
        channel=channel,
    )


def build_mariadb_query_from_param(param, values):
    definition = FILTER_DEFINITIONS[param]
    return build_mariadb_set_query(definition['mariadb_name'], values)


def _empty_filters():
    return dict((name, []) for name in FILTER_DEFINITIONS)


def _split_filter_rule(value):
    if not value:
        return []

    return [item.strip() for item in value.split(',')]


def _build_mysql_filter_values(value_kind, values):
    normalized = normalize_filter_values(values)
    if not normalized:
        return ''

    if value_kind == 'pattern':
        return ','.join(_quote_sql_string(value) for value in normalized)

    return ','.join(mysql_quote_identifier(value, value_kind) for value in normalized)


def _quote_sql_string(value):
    return "'%s'" % value.replace('\\', '\\\\').replace("'", "''")


def _uses_database_table_syntax(value):
    database_name, separator, table_name = value.partition('.')
    return bool(separator and database_name and table_name and '.' not in table_name)


def get_mysql_filter_rows(cursor, channel=''):
    if channel:
        cursor.execute(
            'SELECT CHANNEL_NAME, FILTER_NAME, FILTER_RULE '
            'FROM performance_schema.replication_applier_filters WHERE CHANNEL_NAME = %s',
            (channel,),
        )
    else:
        cursor.execute(
            'SELECT FILTER_NAME, FILTER_RULE FROM performance_schema.replication_applier_global_filters'
        )

    return cursor.fetchall()


def supports_mariadb_connection_name(server_version):
    return LooseVersion(server_version) >= LooseVersion('10.0.1')


def get_mariadb_filter_values(module, cursor, server_version, connection_name=''):
    if connection_name and not supports_mariadb_connection_name(server_version):
        module.fail_json(msg='connection_name requires MariaDB 10.0.1 or newer')

    if connection_name:
        replica_term = 'REPLICA' if LooseVersion(server_version) >= LooseVersion('10.5.1') else 'SLAVE'
        cursor.execute('SHOW %s %s STATUS' % (replica_term, _quote_sql_string(connection_name)))
        rows = cursor.fetchall()
        if rows:
            row = rows[0]
            return {
                'replicate_do_db': row.get('Replicate_Do_DB', ''),
                'replicate_ignore_db': row.get('Replicate_Ignore_DB', ''),
                'replicate_do_table': row.get('Replicate_Do_Table', ''),
                'replicate_ignore_table': row.get('Replicate_Ignore_Table', ''),
                'replicate_wild_do_table': row.get('Replicate_Wild_Do_Table', ''),
                'replicate_wild_ignore_table': row.get('Replicate_Wild_Ignore_Table', ''),
            }
        return dict((param, '') for param in FILTER_DEFINITIONS)

    variables = {}
    for definition in FILTER_DEFINITIONS.values():
        variable_name = definition['mariadb_name']
        cursor.execute('SELECT @@GLOBAL.%s AS Value' % variable_name)
        rows = cursor.fetchall()
        variables[variable_name] = _extract_variable_value(rows)

    return variables


def _extract_variable_value(rows):
    if not rows:
        return ''

    row = rows[0]
    if isinstance(row, dict):
        return row.get('Value', row.get('VALUE', ''))

    if len(row) == 1:
        return row[0]

    return row[1]


class MySQLReplicationFilter(object):
    def __init__(self, module, cursor, server_implementation, server_version):
        self.module = module
        self.cursor = cursor
        self.server_implementation = server_implementation
        self.server_version = server_version

    def apply(self, desired_filters, channel='', connection_name=''):
        self.validate_filters(desired_filters)
        self.validate_target(channel, connection_name)

        if self.server_implementation == 'mysql':
            def build_query(param, values):
                return build_mysql_query_from_param(param, values, channel=channel)

            planned = plan_filter_changes(
                desired_filters,
                normalize_mysql_filter_rows(get_mysql_filter_rows(self.cursor, channel=channel)),
                build_query,
            )
        elif self.server_implementation == 'mariadb':
            planned = plan_filter_changes(
                desired_filters,
                normalize_mariadb_filter_values(get_mariadb_filter_values(
                    self.module,
                    self.cursor,
                    self.server_version,
                    connection_name=connection_name,
                )),
                build_mariadb_query_from_param,
            )
        else:
            self.module.fail_json(
                msg='mysql_replication_filter supports only MySQL or MariaDB, got "%s"' % self.server_implementation
            )

        if not self.module.check_mode:
            self.execute_queries(planned['queries'], connection_name=connection_name)

        return {
            'changed': planned['changed'],
            'queries': [query['display'] for query in planned['queries']],
            'filters': planned['filters'],
        }

    def execute_queries(self, queries, connection_name=''):
        if self.server_implementation == 'mariadb' and connection_name:
            self.cursor.execute('SET @@default_master_connection = %s', (connection_name,))
            self.cursor.fetchall()

        try:
            for query in queries:
                if query['params']:
                    self.cursor.execute(query['sql'], query['params'])
                else:
                    self.cursor.execute(query['sql'])
                self.cursor.fetchall()
        finally:
            if self.server_implementation == 'mariadb' and connection_name:
                self.cursor.execute('SET @@default_master_connection = %s', ('',))
                self.cursor.fetchall()

    def validate_filters(self, desired_filters):
        for param, values in desired_filters.items():
            if values is None:
                continue
            check_input(self.module, values)
            values_with_commas = [value for value in values if ',' in value]
            if values_with_commas:
                self.module.fail_json(
                    msg='replication filter values cannot contain commas: %s' % ', '.join(values_with_commas)
                )
            if FILTER_DEFINITIONS[param]['value_kind'] in ('table', 'pattern'):
                invalid_values = [value for value in values if not _uses_database_table_syntax(value)]
                if invalid_values:
                    self.module.fail_json(
                        msg='%s values must use database.table syntax: %s' % (param, ', '.join(invalid_values))
                    )

    def validate_target(self, channel, connection_name):
        check_input(self.module, channel, connection_name)

        if self.server_implementation == 'mysql' and connection_name:
            self.module.fail_json(msg='connection_name is supported only for MariaDB')

        if self.server_implementation == 'mariadb' and channel:
            self.module.fail_json(msg='channel is supported only for MySQL')


def main():
    argument_spec = mysql_common_argument_spec()
    for name in FILTER_DEFINITIONS:
        argument_spec[name] = dict(type='list', elements='str')

    argument_spec.update(
        channel=dict(type='str'),
        connection_name=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[['channel', 'connection_name']],
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    desired_filters = dict((name, module.params[name]) for name in FILTER_DEFINITIONS)
    if not any(values is not None for values in desired_filters.values()):
        module.fail_json(msg='at least one replication filter option must be provided')

    connect_timeout = module.params['connect_timeout']
    login_user = module.params['login_user']
    login_password = module.params['login_password']
    ssl_cert = module.params['client_cert']
    ssl_key = module.params['client_key']
    ssl_ca = module.params['ca_cert']
    check_hostname = module.params['check_hostname']
    config_file = module.params['config_file']

    try:
        cursor, _db_conn = mysql_connect(
            module,
            login_user,
            login_password,
            config_file,
            ssl_cert,
            ssl_key,
            ssl_ca,
            None,
            cursor_class='DictCursor',
            connect_timeout=connect_timeout,
            check_hostname=check_hostname,
        )
    except Exception as e:
        module.fail_json(
            msg=(
                "unable to connect to database, check login_user and "
                "login_password are correct or %s has the credentials. "
                "Exception message: %s" % (config_file, to_native(e))
            )
        )

    executor = MySQLReplicationFilter(
        module,
        cursor,
        get_server_implementation(module, cursor),
        get_server_version(cursor),
    )

    try:
        module.exit_json(
            **executor.apply(
                desired_filters,
                channel=module.params['channel'],
                connection_name=module.params['connection_name'],
            )
        )
    except ValueError as e:
        module.fail_json(msg='invalid replication filter request: %s' % to_native(e))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_cellular_data_profiles_assignments_info
short_description: Information module for organizations _devices _cellular _data _profiles
  _assignments
description:
  - Get all organizations _devices _cellular _data _profiles _assignments. - > List
    Cellular Data Management Profile assignments in this organization. Returns paginated
    device-to-profile assignments and supports filtering by profile IDs and device
    serials.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  profileIds:
    description:
      - ProfileIds query parameter. Optional parameter to find assignments by Profile
        IDs. Maximum 1000 profile IDs.
    elements: str
    type: list
  serials:
    description:
      - Serials query parameter. Optional parameter to find assignments by Device
        Serials. Maximum 1000 serials.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesCellularDataProfilesAssignments
    description: Complete reference of the getOrganizationDevicesCellularDataProfilesAssignments
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-cellular-data-profiles-assignments
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_cellular_data_profiles_assignments,
  - Paths used are
    get /organizations/{organizationId}/devices/cellular/data/profiles/assignments,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _cellular _data _profiles _assignments
  cisco.meraki.organizations_devices_cellular_data_profiles_assignments_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    profileIds: []
    serials: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "assignmentId": "string",
          "profile": {
            "id": "string"
          },
          "device": {
            "serial": "string"
          }
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_stacks
short_description: Resource module for networks _switch _stacks
description:
  - Manage operations create, update and delete of the resource networks _switch _stacks.
  - Create a switch stack.
  - Delete a stack. - > Update a switch stack. At least one of 'name' or 'members'
    must be provided. If 'members' is provided, it replaces the entire stack membership.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  members:
    description: The complete list of switches that should be in the stack. Minimum
      2 and maximum 8 members. Omitting this field leaves stack membership unchanged.
    elements: dict
    suboptions:
      serial:
        description: Serial number of a switch to include in the stack.
        type: str
    type: list
  name:
    description: The name of the new stack.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  serials:
    description: An array of switch serials to be added into the new stack.
    elements: str
    type: list
  switchStackId:
    description: SwitchStackId path parameter. Switch stack ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchStack
    description: Complete reference of the createNetworkSwitchStack API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-stack
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchStack
    description: Complete reference of the deleteNetworkSwitchStack API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-stack
  - name: Cisco Meraki documentation for switch updateNetworkSwitchStack
    description: Complete reference of the updateNetworkSwitchStack API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-stack
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_stack,
    switch.Switch.delete_network_switch_stack,
    switch.Switch.update_network_switch_stack,
  - Paths used are
    post /networks/{networkId}/switch/stacks,
    delete /networks/{networkId}/switch/stacks/{switchStackId},
    put /networks/{networkId}/switch/stacks/{switchStackId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_stacks:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    name: A cool stack
    networkId: string
    serials:
      - QBZY-XWVU-TSRQ
      - QBAB-CDEF-GHIJ
- name: Update by id
  cisco.meraki.networks_switch_stacks:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    members:
      - serial: QBZY-XWVU-TSRQ
    name: A cool stack
    networkId: string
    switchStackId: string
- name: Delete by id
  cisco.meraki.networks_switch_stacks:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    switchStackId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "serials": [
        "string"
      ],
      "workflowId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_cellular_uplinks_bands_by_device_info
short_description: Information module for organizations _devices _cellular _uplinks
  _bands _by _device
description:
  - Get all organizations _devices _cellular _uplinks _bands _by _device. - > List
    the latest cellular uplink signal information for devices in an organization.
    Data is refreshed at most once every 90 minutes.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  serials:
    description:
      - Serials query parameter. Optional parameter to filter the results by device
        serials. Maximum 1000 serials.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesCellularUplinksBandsByDevice
    description: Complete reference of the getOrganizationDevicesCellularUplinksBandsByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-cellular-uplinks-bands-by-device
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_cellular_uplinks_bands_by_device,
  - Paths used are
    get /organizations/{organizationId}/devices/cellular/uplinks/bands/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _cellular _uplinks _bands _by _device
  cisco.meraki.organizations_devices_cellular_uplinks_bands_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serials: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "bySlot": [
            {
              "slot": "string",
              "bySignalType": [
                {
                  "type": "string",
                  "masked": [
                    "string"
                  ],
                  "enabled": [
                    "string"
                  ],
                  "supported": [
                    "string"
                  ]
                }
              ]
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# Copyright (c) 2026 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: infomaniak_dns_zone_info

short_description: Retrieve zone information in Infomaniak DNS service

version_added: 4.1.0

description:
  - Retrieves zone information in Infomaniak DNS service.
extends_documentation_fragment:
  - community.dns._infomaniak
  - community.dns._module_zone_info
  - community.dns._attributes
  - community.dns._attributes.actiongroup_infomaniak
  - community.dns._attributes.info_module
  - community.dns._attributes.idempotent_not_modify_state
  - community.dns._zone_name_id.name_only_query

author:
  - Felix Fontein (@felixfontein)
"""

EXAMPLES = r"""
- name: Retrieve details for foo.com zone
  community.dns.infomaniak_dns_zone_info:
    zone: foo.com
    infomaniak_token: access_token
  register: rec
"""

RETURN = r"""
zone_name:
  description: The name of the zone.
  type: int
  returned: success
  sample: example.com

zone_id:
  description:
    - The ID of the zone.
    - Note that Infomaniak's API does not support accessing by zone ID,
      whence we use the zone's name as the "ID" for purposes of the collection's interface.
      The actual zone ID can be taken from RV(zone_info.id).
  type: str
  returned: success
  sample: foo.com

zone_info:
  description:
    - Extra information returned by the API.
  type: dict
  returned: success
  contains:
    dnssec:
      description:
        - Information on DNSSEC.
      type: dict
      contains:
        is_enabled:
          description:
            - Whether DNSSEC is enabled for the domain.
          type: bool
          sample: true
      sample:
        is_enabled: true
    id:
      description:
        - The actual zone ID. See RV(zone_id) for more information.
      type: int
      sample: 1234
    nameservers:
      description:
        - List of nameservers the zone should have for using Infomaniak's DNS.
      type: list
      elements: str
      sample:
        - ns1.infomaniak.com
        - ns2.infomaniak.com
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._http import (
    ModuleHTTPHelper,
)
from ansible_collections.community.dns.plugins.module_utils._infomaniak.api import (
    create_infomaniak_api,
    create_infomaniak_argument_spec,
    create_infomaniak_provider_information,
)
from ansible_collections.community.dns.plugins.module_utils._module.zone_info import (
    create_module_argument_spec,
    run_module,
)


def main() -> None:
    provider_information = create_infomaniak_provider_information()
    argument_spec = create_infomaniak_argument_spec()
    argument_spec.merge(
        create_module_argument_spec(provider_information=provider_information)
    )
    module = AnsibleModule(supports_check_mode=True, **argument_spec.to_kwargs())
    option_provider = ModuleOptionProvider(module)
    run_module(
        module,
        lambda: create_infomaniak_api(option_provider, ModuleHTTPHelper(module)),
        provider_information=create_infomaniak_provider_information(
            option_provider=option_provider
        ),
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_appliance_dns_local_records
short_description: Resource module for organizations _appliance _dns _local _records
description:
  - Manage operations create, update and delete of the resource organizations _appliance
    _dns _local _records.
  - Create a new local DNS record.
  - Deletes a local DNS record.
  - Updates a local DNS record.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  address:
    description: IP for the DNS record.
    type: str
  hostname:
    description: Hostname for the DNS record.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  profile:
    description: The profile the DNS record is associated with.
    suboptions:
      id:
        description: Profile ID.
        type: str
    type: dict
  recordId:
    description: RecordId path parameter. Record ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createOrganizationApplianceDnsLocalRecord
    description: Complete reference of the createOrganizationApplianceDnsLocalRecord
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-local-record
  - name: Cisco Meraki documentation for appliance deleteOrganizationApplianceDnsLocalRecord
    description: Complete reference of the deleteOrganizationApplianceDnsLocalRecord
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-appliance-dns-local-record
  - name: Cisco Meraki documentation for appliance updateOrganizationApplianceDnsLocalRecord
    description: Complete reference of the updateOrganizationApplianceDnsLocalRecord
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-dns-local-record
notes:
  - SDK Method used are
    appliance.Appliance.create_organization_appliance_dns_local_record,
    appliance.Appliance.delete_organization_appliance_dns_local_record,
    appliance.Appliance.update_organization_appliance_dns_local_record,
  - Paths used are
    post /organizations/{organizationId}/appliance/dns/local/records,
    delete /organizations/{organizationId}/appliance/dns/local/records/{recordId},
    put /organizations/{organizationId}/appliance/dns/local/records/{recordId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_appliance_dns_local_records:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    address: 10.1.1.0
    hostname: www.test.com
    organizationId: string
    profile:
      id: '1'
- name: Update by id
  cisco.meraki.organizations_appliance_dns_local_records:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    address: 10.1.1.0
    hostname: www.test.com
    organizationId: string
    profile:
      id: '1'
    recordId: string
- name: Delete by id
  cisco.meraki.organizations_appliance_dns_local_records:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    organizationId: string
    recordId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  sample: >
    [
      {
        "recordId": "string",
        "hostname": "string",
        "address": "string",
        "profile": {
          "id": "string"
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_routing_table_lookups
short_description: Resource module for devices _live _tools _routing _table _lookups
description:
  - Manage operation create of the resource devices _live _tools _routing _table _lookups.
    - > Enqueue a job to perform a routing table lookup request for a device. The
    routing table lookup request fetches a specific set of routes based on filters.
    Any combination of search filters can be applied. Only Cisco Secure Routers are
    supported.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  callback:
    description: Details for the callback. Please include either an httpServerId OR
      url and sharedSecret.
    suboptions:
      httpServer:
        description: The webhook receiver used for the callback webhook.
        suboptions:
          id:
            description: The webhook receiver ID that will receive information. If
              specifying this, please leave the url and sharedSecret fields blank.
            type: str
        type: dict
      payloadTemplate:
        description: The payload template of the webhook used for the callback.
        suboptions:
          id:
            description: The ID of the payload template. Defaults to 'wpt_00005' for
              the Callback (included) template.
            type: str
        type: dict
      sharedSecret:
        description: A shared secret that will be included in the requests sent to
          the callback URL. It can be used to verify that the request was sent by
          Meraki. If using this field, please also specify an url.
        type: str
      url:
        description: The callback URL for the webhook target. If using this field,
          please also specify a sharedSecret.
        type: str
    type: dict
  destination:
    description: The destination IP or subnet to lookup.
    suboptions:
      address:
        description: The destination IP address to find a longest prefix match route.
        type: str
      subnet:
        description: The destination subnet in CIDR format to find matching routes.
        type: str
    type: dict
  nextHop:
    description: The next hop to lookup.
    suboptions:
      address:
        description: The IP address of the next hop.
        type: str
    type: dict
  serial:
    description: Serial path parameter.
    type: str
  type:
    description: The type of route defined.
    type: str
  vpn:
    description: VPN related search criteria.
    suboptions:
      peer:
        description: The VPN peer to lookup.
        suboptions:
          id:
            description: The Network ID of the VPN peer.
            type: str
        type: dict
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices createDeviceLiveToolsRoutingTableLookup
    description: Complete reference of the createDeviceLiveToolsRoutingTableLookup
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-live-tools-routing-table-lookup
notes:
  - SDK Method used are
    devices.Devices.create_device_live_tools_routing_table_lookup,
  - Paths used are
    post /devices/{serial}/liveTools/routingTable/lookups,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_live_tools_routing_table_lookups:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    callback:
      httpServer:
        id: aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
      payloadTemplate:
        id: wpt_2100
      sharedSecret: secret
      url: https://webhook.site/28efa24e-f830-4d9f-a12b-fbb9e5035031
    destination:
      address: 192.168.0.10
      subnet: 192.168.0.0/24
    nextHop:
      address: 10.10.0.1
    serial: string
    type: static
    vpn:
      peer:
        id: N_12345678
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "lookupId": "string",
      "url": "string",
      "request": {
        "serial": "string",
        "type": "string",
        "destination": {
          "address": "string",
          "subnet": "string"
        },
        "nextHop": {
          "address": "string"
        },
        "vpn": {
          "peer": {
            "id": "string"
          }
        }
      },
      "status": "string",
      "callback": {
        "id": "string",
        "url": "string",
        "status": "string"
      }
    }
"""



# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: reverse_lookup
author: Felix Fontein (@felixfontein)
short_description: Reverse-look up IP addresses
version_added: 3.1.0
requirements:
  - dnspython >= 2.0.0
description:
  - Look up hostnames for IP addresses using DNS reverse lookup.
options:
  _terms:
    description:
      - IP address(es) to look up.
    type: list
    elements: str
    required: true
  query_retry:
    description:
      - Number of retries for DNS query timeouts.
    type: int
    default: 3
  query_timeout:
    description:
      - Timeout per DNS query in seconds.
    type: float
    default: 10
  server:
    description:
      - The DNS server(s) to use to look up the result. Must be a list of one or more IP addresses.
      - By default, the system's standard resolver is used.
    type: list
    elements: str
  servfail_retries:
    description:
      - How often to retry on SERVFAIL errors.
    type: int
    default: 0
notes:
  - Note that when using this lookup plugin with V(lookup(\)), and the result is a one-element list, Ansible simply returns
    the one element not as a list. Since this behavior is surprising and can cause problems, it is better to use V(query(\))
    instead of V(lookup(\)). See the examples and also R(Forcing lookups to return lists, query) in the Ansible documentation.
"""

EXAMPLES = r"""
- name: Look up hostname of IPv4 address
  ansible.builtin.debug:
    msg: "{{ query('community.dns.reverse_lookup', '192.168.1.1') }}"

- name: Look up hostname of IPv6 address
  ansible.builtin.debug:
    msg: "{{ query('community.dns.reverse_lookup', '1:2:3::4') }}"
"""

RETURN = r"""
_result:
  description:
    - The hostname(s) returned for the queried IP addresses.
    - If multiple IP addresses are queried in O(_terms), the resulting lists have been concatenated.
  type: list
  elements: str
  sample:
    - example.com
    - example.org
"""

import ipaddress
import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleLookupError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.dns.plugins.module_utils._ips import is_ip_address
from ansible_collections.community.dns.plugins.module_utils._resolver import (
    SimpleResolver,
)
from ansible_collections.community.dns.plugins.plugin_utils._resolver import (
    assert_requirements_present as assert_requirements_present_dnspython,
)
from ansible_collections.community.dns.plugins.plugin_utils._resolver import (
    guarded_run,
)

try:
    import dns.resolver
    from dns.rdatatype import RdataType
    from dns.resolver import NXDOMAIN
except ImportError:
    # handled by assert_requirements_present_dnspython
    pass
else:
    RdataType = int  # type: ignore  # noqa: F811


class LookupModule(LookupBase):
    @staticmethod
    def _resolve(
        resolver: SimpleResolver,
        name: str,
        rdtype: RdataType,
        server_addresses: list[str] | None,
    ) -> list[str]:
        def callback() -> list[str]:
            rrset = resolver.resolve(
                name,
                rdtype=rdtype,
                server_addresses=server_addresses,
                nxdomain_is_empty=True,
                target_can_be_relative=False,
                search=False,
            )
            if not rrset:
                return []
            return [to_text(data) for data in rrset]

        return guarded_run(
            callback,
            error_class=AnsibleLookupError,
            server=name,
        )

    @staticmethod
    def _get_resolver(resolver: SimpleResolver, server: str) -> Callable[[], list[str]]:
        def f():
            try:
                return resolver.resolve_addresses(server)
            except NXDOMAIN as exc:
                raise AnsibleLookupError(
                    f"Nameserver {server} does not exist ({exc})"
                ) from None

        return f

    def run(
        self, terms: list[t.Any], variables: t.Any | None = None, **kwargs: t.Any
    ) -> list[str]:
        assert_requirements_present_dnspython("community.dns.reverse_lookup", "lookup")

        self.set_options(var_options=variables, direct=kwargs)

        resolver = SimpleResolver(
            timeout=self.get_option("query_timeout"),
            timeout_retries=self.get_option("query_retry"),
            servfail_retries=self.get_option("servfail_retries"),
        )

        server_addresses: list[str] | None = None
        if self.get_option("server"):
            server_addresses = []
            servers: list[str] = self.get_option("server")
            for server in servers:
                if is_ip_address(server):
                    server_addresses.append(server)
                    continue
                server_addresses.extend(
                    guarded_run(
                        self._get_resolver(resolver, server),
                        error_class=AnsibleLookupError,
                        server=server,
                    )
                )

        ip_adresses = []
        for ip_address in terms:
            try:
                ipaddr = ipaddress.ip_address(to_text(ip_address))
                name = ipaddr.reverse_pointer
                if not name.endswith("."):
                    name += "."
                else:
                    pass  # pragma: no cover
                ip_adresses.append(name)
            except Exception as e:
                raise AnsibleLookupError(
                    f"Cannot parse IP address {ip_address!r}: {e}"
                ) from e

        result = []
        for name in ip_adresses:
            result.extend(
                self._resolve(resolver, name, dns.rdatatype.PTR, server_addresses)
            )
        return result



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_serviceendpointpolicy
version_added: "3.9.0"
short_description: Managed the service endpoint policy
description:
    - Create, update or delete the service endpoint policy.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    name:
        description:
            - The name of the service endpoint policy name.
        type: str
        required: true
    location:
        description:
            - THe resource's location.
        type: str
    state:
        description:
            - Set to C(present) to create or update the service endpoint policy.
            - Set to C(absent) to remove the service endpoint policy.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - magodo (@magodo)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new service endpoint policy
  azure.azcollection.azure_rm_serviceendpointpolicy:
    resource_group: testRG
    name: policyname
    locaiton: eastus
    tags:
      key: value

- name: Delete the service endpoint policy
  azure.azcollection.azure_rm_serviceendpointpolicy:
    resource_group: testRG
    name: policyname
    state: absent
'''

RETURN = '''
serviceendpointpolicypolicy:
    description:
        - The fact of the service endpoint policys in a service end point policy.
    type: complex
    returned: always
    contains:
        id:
            description:
                - Fully qualified ID of the resource.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/TestRG/providers/Microsoft.Network/serviceEndpointPolicies/testpolicy"
        name:
            description:
                - The name of the resource that is unique within a resource group.
            type: str
            returned: always
            sample: testpolicy
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: "49e15a7b-1bff-46ba-b962-7a11e75cb729"
        resource_group:
            description:
                - The name of the resource group.
            type: str
            returned: always
            sample: TestRG
        location:
            description:
                - The resource's location.
            type: str
            returned: always
            sample: eastus
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: Microsoft.Network/serviceEndpointPolicies
        service_endpoint_policy_definitions:
            description:
                - A collection of service endpoint policy definitions of the service endpoint policy.
            type: list
            returned: always
            sample: []
        provisioning_state:
            description:
                - The provisioning state of the service endpoint policy resource.
            type: str
            returned: always
            sample: Successded
        resource_guid:
            description:
                - The resource GUID property of the service endpoint policy resource.
            type: str
            returned: always
            sample: 77ee69ee-4152-4271-bfc0-1d431b7b7d28
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
try:
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMServiceEndpointPolicy(AzureRMModuleBase):
    """Information class for an Azure RM Data Collection Rules"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.tags = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            serviceendpointpolicypolicy=dict()
        )

        super(AzureRMServiceEndpointPolicy, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                           supports_check_mode=True,
                                                           supports_tags=True,
                                                           facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        response = self.get_policy(self.name)
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        changed = False
        if self.state == 'present':
            if response:
                update_tags, self.tags = self.update_tags(response.get('tags', dict()))
                if update_tags:
                    changed = True
                    if not self.check_mode:
                        response = self.update_tag()
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_or_update()
        else:
            if response:
                changed = True
                if self.check_mode:
                    self.log("The service endpoint policy already exist, will be delete")
                else:
                    response = self.delete()
            else:
                if self.check_mode:
                    self.log("There is no service endpoint policy.")

        self.results['changed'] = changed
        self.results['serviceendpointpolicypolicy'] = response
        return self.results

    def get_policy(self, name):
        '''
        Gets the specified service endpoint policy
        '''
        response = None
        try:
            response = self.network_client.service_endpoint_policies.get(self.resource_group, self.name)
        except Exception:
            self.log("Could not get the specified service endpoint policy {0}".format(self.name))
        return self.format_item(response) if response else None

    def create_or_update(self):
        '''
        Create or update the service endpoint policy.
        '''
        response = None
        try:
            response = self.network_client.service_endpoint_policies.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                            service_endpoint_policy_name=self.name,
                                                                                            parameters=dict(location=self.location,
                                                                                                            tags=self.tags))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as ex:
            self.fail("Could not create or update the service endpoint policy, Exception as {0}".format(ex))
        return self.format_item(response) if response else None

    def update_tag(self):
        '''
        Update the service endpoint policy tags.
        '''
        response = None
        try:
            response = self.network_client.service_endpoint_policies.update_tags(resource_group_name=self.resource_group,
                                                                                 service_endpoint_policy_name=self.name,
                                                                                 parameters=dict(tags=self.tags))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as ex:
            self.fail("Could not create or update the service endpoint policy, Exception as {0}".format(ex))
        return self.format_item(response) if response else None

    def delete(self):
        '''
        Delete the service endpoint policy.
        '''
        try:
            response = self.network_client.service_endpoint_policies.begin_delete(self.resource_group, self.name)
            if isinstance(response, LROPoller):
                self.get_poller_result(response)
        except Exception as ex:
            self.fail("Could not delete the service endpoint policy, Exception as {0}".format(ex))

    def format_item(self, item):
        response = item.as_dict()
        response['resource_group'] = self.resource_group
        return response


def main():
    """Main execution"""
    AzureRMServiceEndpointPolicy()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_schedule_info
version_added: "3.19.0"
short_description: List ML Schedule resources
description:
    - List ML Schedule resources.
options:
    name:
        description:
            - Name of the Azure ML schedule.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    list_type:
        description:
            - Specify if you want enabled, disabled or all schedules.
        type: str
        choices:
            - enabled
            - disabled
            - all

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: List ML Schedule by name
  azure.azcollection.azure_rm_ml_schedule_info:
    name: MySchedule
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List Disabled ML Schedules
  azure.azcollection.azure_rm_ml_schedule_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    list_type: disabled
'''

RETURN = '''
ml_schedules:
    description:
        - Schedules that match the query.
    returned: always
    type: dict
    sample: [
      {
        "create_job": {
            "compute": "azureml:cpu-cluster",
            "display_name": "hello_pipeline_abc",
            "experiment_name": "Default",
            "inputs": {
                "hello_string_top_level_input": "hello world"
            },
            "jobs": {
                "a": {
                    "component": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-schedule/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/azureml_anonymous/versions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "inputs": {
                        "hello_string": {
                            "path": "${{ '{{' }}parent.inputs.hello_string_top_level_input{{ '}}' }}"
                        }
                    },
                    "type": "command"
                },
                "b": {
                    "component": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-schedule/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/azureml_anonymous/versions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "type": "command"
                },
                "c": {
                    "component": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-schedule/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/azureml_anonymous/versions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "inputs": {
                        "world_input": {
                            "path": "${{ '{{' }}parent.jobs.b.outputs.world_output{{ '}}' }}"
                        }
                    },
                    "type": "command"
                }
            },
            "status": "NotStarted",
            "type": "pipeline"
        },
        "creation_context": {
            "created_at": "2026-05-14T16:46:52.141013+00:00",
            "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "created_by_type": "User",
            "last_modified_at": "2026-05-14T16:47:49.168130+00:00",
            "last_modified_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "last_modified_by_type": "User"
        },
        "description": "a simple hourly cron job schedule",
        "display_name": "Simple cron job schedule",
        "is_enabled": false,
        "name": "schedulexxxxxxxxxx",
        "provisioning_state": "Succeeded",
        "trigger": {
            "expression": "1 * * * *",
            "start_time": "2022-07-10 10:00:00",
            "time_zone": "Pacific Standard Time",
            "type": "cron"
        }
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLScheduleInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            list_type=dict(
                type='str',
                choices=['disabled', 'enabled', 'all'],
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_schedules=[]
        )

        mutually_exclusive = [('name', 'list_type')]

        super(AzureRMMLScheduleInfo, self).__init__(
            self.module_arg_spec,
            supports_tags=False,
            supports_check_mode=True,
            facts_module=True,
            mutually_exclusive=mutually_exclusive,
        )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.schedules.get(name=self.name)
                ml_schedules = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_schedules = []
        else:
            list_view_type = self.get_schedule_list_view_type(self.list_type)
            results = self.client.schedules.list(
                list_view_type=list_view_type
            )
            ml_schedules = [self.entity_to_dict(self.client.schedules.get(x.name)) for x in results]

        self.results['ml_schedules'] = ml_schedules
        return self.results


def main():
    AzureRMMLScheduleInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2019, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


def binary_exp_mod(f: int, e: int, *, m: int) -> int:
    """Computes f^e mod m in O(log e) multiplications modulo m."""
    # Compute len_e = floor(log_2(e))
    len_e = -1
    x = e
    while x > 0:
        x >>= 1
        len_e += 1
    # Compute f**e mod m
    result = 1
    for k in range(len_e, -1, -1):
        result = (result * result) % m
        if ((e >> k) & 1) != 0:
            result = (result * f) % m
    return result


def simple_gcd(a: int, b: int) -> int:
    """Compute GCD of its two inputs."""
    while b != 0:
        a, b = b, a % b
    return a


def quick_is_not_prime(n: int) -> bool:
    """Does some quick checks to see if we can poke a hole into the primality of n.

    A result of `False` does **not** mean that the number is prime; it just means
    that we could not detect quickly whether it is not prime.
    """
    if n <= 2:
        return n < 2
    # The constant in the next line is the product of all primes < 200
    prime_product = 7799922041683461553249199106329813876687996789903550945093032474868511536164700810
    gcd = simple_gcd(n, prime_product)
    if gcd > 1:
        if n < 200 and gcd == n:
            # Explicitly check for all primes < 200
            return n not in (
                2,
                3,
                5,
                7,
                11,
                13,
                17,
                19,
                23,
                29,
                31,
                37,
                41,
                43,
                47,
                53,
                59,
                61,
                67,
                71,
                73,
                79,
                83,
                89,
                97,
                101,
                103,
                107,
                109,
                113,
                127,
                131,
                137,
                139,
                149,
                151,
                157,
                163,
                167,
                173,
                179,
                181,
                191,
                193,
                197,
                199,
            )
        return True
    # TODO: maybe do some iterations of Miller-Rabin to increase confidence
    # (https://en.wikipedia.org/wiki/Miller%E2%80%93Rabin_primality_test)
    return False


def count_bytes(no: int) -> int:
    """
    Given an integer, compute the number of bytes necessary to store its absolute value.
    """
    no = abs(no)
    if no == 0:
        return 0
    return (no.bit_length() + 7) // 8


def count_bits(no: int) -> int:
    """
    Given an integer, compute the number of bits necessary to store its absolute value.
    """
    no = abs(no)
    if no == 0:
        return 0
    return no.bit_length()


def convert_int_to_bytes(no: int, *, count: int | None = None) -> bytes:
    """
    Convert the absolute value of an integer to a byte string in network byte order.

    If ``count`` is provided, it must be sufficiently large so that the integer's
    absolute value can be represented with these number of bytes. The resulting byte
    string will have length exactly ``count``.

    The value zero will be converted to an empty byte string if ``count`` is provided.
    """
    no = abs(no)
    if count is None:
        count = count_bytes(no)
    return no.to_bytes(count, byteorder="big")


def convert_int_to_hex(no: int, *, digits: int | None = None) -> str:
    """
    Convert the absolute value of an integer to a string of hexadecimal digits.

    If ``digits`` is provided, the string will be padded on the left with ``0``s so
    that the returned value has length ``digits``. If ``digits`` is not sufficient,
    the string will be longer.
    """
    no = abs(no)
    value = f"{no:x}"
    if digits is not None and len(value) < digits:
        value = "0" * (digits - len(value)) + value
    return value


def convert_bytes_to_int(data: bytes) -> int:
    """
    Convert a byte string to an unsigned integer in network byte order.
    """
    return int.from_bytes(data, byteorder="big", signed=False)


__all__ = (
    "binary_exp_mod",
    "convert_bytes_to_int",
    "convert_int_to_bytes",
    "convert_int_to_hex",
    "count_bits",
    "count_bytes",
    "quick_is_not_prime",
    "simple_gcd",
)
